# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Contains conversion package from INSPIRE to CDS MARCXML.

>>> from harvestingkit.bibrecord import BibRecordPackage
>>> from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS
>>> bibrecs = BibRecordPackage("inspire.xml")
>>> bibrecs.parse()
>>> cds_xml = Inspire2CDS.convert(bibrecs.get_records())
"""

from __future__ import print_function

from datetime import datetime

from ..bibrecord import (record_get_field_instances,
                         record_add_field,
                         record_get_field_values,
                         record_delete_field,
                         record_delete_fields,
                         field_get_subfield_instances,
                         field_swap_subfields,
                         field_get_subfields)

from ..utils import (
    convert_date_from_iso_to_human,
    return_letters_from_string
)

from .base import MARCXMLConversion


class Inspire2CDS(MARCXMLConversion):

    """Convert INSPIRE to CDS."""

    # By setting the class variable here, we run it only once a session
    kbs = MARCXMLConversion.load_config("inspire", "cds")

    def __init__(self, bibrec, strip_fields_list=None):
        """Create."""
        super(Inspire2CDS, self).__init__(bibrec, strip_fields_list)
        self.collections = set([])
        self.tag_as_cern = False
        self.collection_base = {
            "PROCEEDINGS": "43",
            "ARTICLE": "13",
            "THESIS": "14",
            "PREPRINT": "11",
            "ANNOUNCEMENT": "41",
        }
        self.recid = None
        self.conference_recid = None
        self.conference_codes = None
        self.conference_pages = None

    def get_cnums(self):
        return record_get_field_values(self.record, tag="773", code="w")

    def update_conference_info(self):
        if self.conference_recid:
            subfields = [("b", str(self.conference_recid))]
            if self.conference_codes:
                subfields.append(("n", self.conference_codes[0]))
            if self.conference_pages:
                subfields.append(("k", self.conference_pages[0]))
            record_add_field(self.record, "962", subfields=subfields)

    def update_conference_111(self):
        scn_111_fields = record_get_field_instances(self.record, '111')
        record_delete_fields(self.record, tag="111")
        new_fields = []
        for field in scn_111_fields:
            subs = field_get_subfields(field)
            new_subs = []
            month = ""
            day = ""
            year = ""
            place = ""
            start_date = ""
            end_date = ""
            for code, values in subs.iteritems():
                if not values:
                    continue
                if code == 'g':
                    record_add_field(self.record, tag="035",
                                     subfields=[('9', 'Inspire-CNUM'),
                                                ('a', values[0])])
                elif code == "x":
                    date = values[0].split('-')
                    if len(date) >= 1:
                        year = date[0]
                    if len(date) >= 2:
                        month = date[1]
                    if len(date) >= 3:
                        day = date[2]
                    date_stripped = "".join(date)
                    new_subs.append(("9", date_stripped))
                    if len(date_stripped) == 8:
                        start_date = date_stripped

                    if year:
                        new_subs.append(("f", year))
                        record_add_field(self.record, tag="260",
                                         subfields=[('c', year)])
                elif code == "y":
                    date_stripped = "".join(values[0].split('-'))
                    new_subs.append(("z", date_stripped))
                    if len(date_stripped) == 8:
                        end_date = date_stripped
                elif code == "c":
                    place_parts = [v.strip() for v in values[0].split(',')]
                    if place_parts:
                        place = place_parts[0].lower()
                    new_subs.append((code, values[0]))
                else:
                    new_subs.append((code, values[0]))

            if new_subs:
                if day and month and year and place:
                    new_subs.append(
                        ('g', "{0}{1}{2}{3}".format(place, year, month, day))
                    )
                if start_date and end_date:
                    if end_date[4:6] == start_date[4:6]:
                        # same month
                        date_format = "{0}-{1}".format(
                            start_date[-2:],
                            datetime.strptime(end_date, '%Y%m%d').strftime('%d %b %Y')
                        )
                    else:
                        date_format = "{0}-{1}".format(
                            datetime.strptime(start_date, '%Y%m%d').strftime('%d %b'),
                            datetime.strptime(end_date, '%Y%m%d').strftime('%d %b %Y')
                        )
                    new_subs.append(
                        ('d', date_format)
                    )
                record_add_field(self.record,
                                 tag="111",
                                 subfields=new_subs)

    def update_conference_links(self):
        scn_856_fields = record_get_field_instances(self.record,
                                                    tag='856',
                                                    ind1="4")
        new_fields = []
        for field in scn_856_fields:
            subs = field_get_subfields(field)
            new_subs = []
            if 'y' not in subs:
                new_subs.append(('y', 'Conference home page'))
            for code, values in subs.iteritems():
                new_subs.append((code, values[0]))
            new_fields.append(new_subs)

        record_delete_fields(self.record, tag="856")
        for field in new_fields:
            record_add_field(self.record, tag="856", ind1="4", subfields=field)

    def get_record(self):
        """Override the base."""
        self.recid = self.get_recid()
        self.remove_controlfields()
        self.update_system_numbers()
        self.add_systemnumber("Inspire", recid=self.recid)
        self.add_control_number("003", "SzGeCERN")
        self.update_collections()
        self.update_languages()
        self.update_authors()
        self.update_journals()
        self.update_subject_categories("INSPIRE", "SzGeCERN", "categories_cds")
        self.update_pagenumber()
        self.update_notes()
        self.update_experiments()
        self.update_isbn()
        self.update_dois()
        self.update_links_and_ffts()
        self.update_date()
        self.update_date_year()
        self.update_hidden_notes()
        self.update_oai_info()
        self.update_cnum()
        self.update_conference_info()
        self.update_collaboration()
        self.update_542()

        self.fields_list = [
            "909", "541", "961",
            "970", "690", "695",
            "981", "999",
        ]
        self.strip_fields()

        if "ANNOUNCEMENT" in self.collections:
            self.update_conference_111()
            self.update_conference_links()
            record_add_field(self.record, "690", ind1="C", subfields=[("a", "CONFERENCE")])

        if "THESIS" in self.collections:
            self.update_thesis_information()
            # disabled on request of the CERN Library (SNOW: RQF1302222)
            # self.update_thesis_supervisors()

        if "PROCEEDINGS" in self.collections:
            # Special proceeding syntax
            self.update_title_to_proceeding()
            self.update_author_to_proceeding()
            record_add_field(self.record, "690", ind1="C", subfields=[("a", "CONFERENCE")])

        # 690 tags
        if self.tag_as_cern:
            record_add_field(self.record, "690", ind1="C", subfields=[("a", "CERN")])

        return self.record

    def update_oai_info(self):
        """Add the 909 OAI info to 035."""
        for field in record_get_field_instances(self.record, '909', ind1="C", ind2="O"):
            new_subs = []
            for tag, value in field[0]:
                if tag == "o":
                    new_subs.append(("a", value))
                else:
                    new_subs.append((tag, value))
                if value in ["CERN", "CDS", "ForCDS"]:
                    self.tag_as_cern = True
            record_add_field(self.record, '024', ind1="8", subfields=new_subs)
        record_delete_fields(self.record, '909')

    def update_cnum(self):
        """Check if we shall add cnum in 035."""
        if "ConferencePaper" not in self.collections:
            cnums = record_get_field_values(self.record, '773', code="w")
            for cnum in cnums:
                cnum_subs = [
                    ("9", "INSPIRE-CNUM"),
                    ("a", cnum)
                ]
                record_add_field(self.record, "035", subfields=cnum_subs)

    def update_hidden_notes(self):
        """Remove hidden notes and tag a CERN if detected."""
        if not self.tag_as_cern:
            notes = record_get_field_instances(self.record,
                                               tag="595")
            for field in notes:
                for dummy, value in field[0]:
                    if value == "CDS":
                        self.tag_as_cern = True
        record_delete_fields(self.record, tag="595")

    def update_system_numbers(self):
        """035 Externals."""
        scn_035_fields = record_get_field_instances(self.record, '035')
        new_fields = []
        for field in scn_035_fields:
            subs = field_get_subfields(field)
            if '9' in subs:
                if subs['9'][0].lower() == "cds" and subs.get('a'):
                    self.add_control_number("001", subs.get('a')[0])
                if subs['9'][0].lower() in ["inspire", "spirestex", "inspiretex", "desy", "cds"]:
                    continue
            new_fields.append(field_get_subfield_instances(field))
        record_delete_fields(self.record, tag="035")
        for field in new_fields:
            record_add_field(self.record, tag="035", subfields=field)

    def update_collections(self):
        """Try to determine which collections this record should belong to."""
        def should_add_article():
            """Return True if the record contains all must have keys."""
            must_have_keys = set(["c", "p", "v", "y"])
            f773 = record_get_field_instances(self.record, '773')
            found_keys = set()
            for field in f773:
                for subfield in field_get_subfields(field):
                    found_keys.add(subfield[0])
            return must_have_keys.issubset(found_keys)

        for value in record_get_field_values(self.record, '980', code='a'):
            v = value.upper()
            if 'NOTE' in v:
                self.collections.add('NOTE')
            if 'THESIS' in v:
                self.collections.add('THESIS')
            if 'PUBLISHED' in v:
                self.collections.add('ARTICLE')
            if 'CONFERENCES' in v:
                self.collections.add('ANNOUNCEMENT')
            if 'PROCEEDINGS' in v:
                self.collections.add('PROCEEDINGS')
            elif 'CONFERENCEPAPER' in v and \
                 "ConferencePaper" not in self.collections:
                self.collections.add('ConferencePaper')
                if "ARTICLE" not in self.collections and should_add_article():
                    self.collections.add('ARTICLE')
                else:
                    self.collections.add('PREPRINT')

            if "HIDDEN" in v:
                self.hidden = True

        # Clear out any existing ones.
        record_delete_fields(self.record, "980")

        if not self.collections:
            self.collections.add('PREPRINT')

        for collection in self.collections:
            record_add_field(self.record,
                             tag='980',
                             subfields=[('a', collection)])
            if collection in self.collection_base:
                subs = [('a', self.collection_base[collection])]
                record_add_field(self.record,
                                 tag='960',
                                 subfields=subs)

    def update_notes(self):
        """Remove INSPIRE specific notes."""
        fields = record_get_field_instances(self.record, '500')
        for field in fields:
            subs = field_get_subfields(field)
            for sub in subs.get('a', []):
                sub = sub.strip()  # remove any spaces before/after
                if sub.startswith("*") and sub.endswith("*"):
                    record_delete_field(self.record, tag="500",
                                        field_position_global=field[4])

    def update_title_to_proceeding(self):
        """Move title info from 245 to 111 proceeding style."""
        titles = record_get_field_instances(self.record,
                                            tag="245")
        for title in titles:
            subs = field_get_subfields(title)
            new_subs = []
            if "a" in subs:
                new_subs.append(("a", subs['a'][0]))
            if "b" in subs:
                new_subs.append(("c", subs['b'][0]))
            record_add_field(self.record,
                             tag="111",
                             subfields=new_subs)
        record_delete_fields(self.record, tag="245")
        record_delete_fields(self.record, tag="246")

    def update_author_to_proceeding(self):
        """Move author info from 245 to 111 proceeding style."""
        titles = record_get_field_instances(self.record,
                                            tag="245")
        for title in titles:
            subs = field_get_subfields(title)
            new_subs = []
            if "a" in subs:
                new_subs.append(("a", subs['a'][0]))
            if "b" in subs:
                new_subs.append(("c", subs['b'][0]))
            record_add_field(self.record,
                             tag="111",
                             subfields=new_subs)
        record_delete_fields(self.record, tag="245")
        record_delete_fields(self.record, tag="246")

    def update_experiments(self):
        """Experiment mapping."""
        # 693 Remove if 'not applicable'
        for field in record_get_field_instances(self.record, '693'):
            subs = field_get_subfields(field)
            experiment_testbeams = subs.get("b", [])
            acc_experiment = subs.get("e", [])
            if not acc_experiment:
                acc_experiment = subs.get("a", [])
                if not acc_experiment:
                    continue
            experiment = acc_experiment[-1]

            # Handle special case of leading experiments numbers NA-050 -> NA 50
            e_suffix = ""
            if "-NA-" in experiment or \
               "-RD-" in experiment or \
               "-WA-" in experiment:
                splitted_experiment = experiment.split("-")
                e_suffix = "-".join(splitted_experiment[2:])
                if e_suffix.startswith("0"):
                    e_suffix = e_suffix[1:]
                experiment = "-".join(splitted_experiment[:2])  # only CERN-NA

            translated_experiment = self.get_config_item(experiment,
                                                         "experiments")
            if not translated_experiment:
                continue
            new_subs = [("b", n) for n in experiment_testbeams]
            if "---" in translated_experiment:
                experiment_a, experiment_e = translated_experiment.split("---")
                new_subs.append(("a", experiment_a.replace("-", " ")))
            else:
                experiment_e = translated_experiment
            new_subs.append(("e", experiment_e.replace("-", " ") + e_suffix))
            record_delete_field(self.record, tag="693",
                                field_position_global=field[4])
            record_add_field(self.record, "693", subfields=new_subs)

    def update_authors(self):
        """100 & 700 punctuate author names."""
        author_names = record_get_field_instances(self.record, '100')
        author_names.extend(record_get_field_instances(self.record, '700'))
        for field in author_names:
            subs = field_get_subfields(field)
            for idx, (key, value) in enumerate(field[0]):
                if key == 'a':
                    field[0][idx] = ('a', value.replace(".", " ").strip())
                elif key == 'v':
                    del field[0][idx]
            if subs.get("u", None) == "CERN":
                self.tag_as_cern = True

    def update_collaboration(self):
        """Add "Collaboration" to the value of the field 710__g."""
        collabs = record_get_field_instances(self.record, '710')
        for field in collabs:
            subs = field_get_subfield_instances(field)
            for idx, (key, value) in enumerate(subs):
                if key == "g" and value and "Collaboration" not in value:
                    field[0][idx] = ("g", value + " Collaboration")

    def update_542(self):
        """Change 542__e -> 542__3 and 542__e:Article to 542__3:publication."""
        fields = record_get_field_instances(self.record, '542')
        for field in fields:
            subs = field_get_subfield_instances(field)
            for idx, (key, value) in enumerate(subs):
                if key != 'e':
                    continue
                # change 'article' -> 'publication'
                v = 'publication' if value.strip().lower() == 'article' else value

                # update '__e'
                field[0][idx] = ("3", v)

    def update_isbn(self):
        """Remove dashes from ISBN."""
        isbns = record_get_field_instances(self.record, '020')
        for field in isbns:
            for idx, (key, value) in enumerate(field[0]):
                if key == 'a':
                    field[0][idx] = ('a', value.replace("-", "").strip())

    def update_dois(self):
        """Remove duplicate BibMatch DOIs."""
        dois = record_get_field_instances(self.record, '024', ind1="7")
        all_dois = {}
        for field in dois:
            subs = field_get_subfield_instances(field)
            subs_dict = dict(subs)
            if subs_dict.get('a'):
                if subs_dict['a'] in all_dois:
                    record_delete_field(self.record, tag='024', ind1='7', field_position_global=field[4])
                    continue
                all_dois[subs_dict['a']] = field

    def update_journals(self):
        """773 journal translations."""
        for field in record_get_field_instances(self.record, '773'):
            subs = field_get_subfield_instances(field)
            new_subs = []
            volume_letter = ""
            journal_name = ""
            for idx, (key, value) in enumerate(subs):
                if key == 'p':
                    journal_name = self.get_config_item(value, "journals", allow_substring=False)
                    # Make sure journal names have the form (dot)(space) (I know it's horrible)
                    journal_name = journal_name.replace('. ', '.').replace('.', '. ').replace('. ,', '.,').strip()
                elif key == 'v':
                    volume_letter = value
                else:
                    new_subs.append((key, value))

            if not journal_name == "PoS":
                # Special handling of journal name and volumes, except PoS
                letter = return_letters_from_string(volume_letter)
                if letter:
                    journal_name = "{0} {1}".format(journal_name, letter)
                    volume_letter = volume_letter.strip(letter)

            if journal_name:
                new_subs.append(("p", journal_name))
            if volume_letter:
                new_subs.append(("v", volume_letter))
            record_delete_field(self.record, tag="773",
                                field_position_global=field[4])
            record_add_field(self.record, "773", subfields=new_subs)

    def update_thesis_supervisors(self):
        """700 -> 701 Thesis supervisors."""
        for field in record_get_field_instances(self.record, '701'):
            subs = list(field[0])
            subs.append(("e", "dir."))
            record_add_field(self.record, '700', subfields=subs)
        record_delete_fields(self.record, '701')

    def update_thesis_information(self):
        """501 degree info - move subfields."""
        fields_501 = record_get_field_instances(self.record, '502')
        for field in fields_501:
            new_subs = []
            for key, value in field[0]:
                if key == 'b':
                    new_subs.append(('a', value))
                elif key == 'c':
                    new_subs.append(('b', value))
                elif key == 'd':
                    new_subs.append(('c', value))
                else:
                    new_subs.append((key, value))
            record_delete_field(self.record, tag="502",
                                field_position_global=field[4])
            record_add_field(self.record, "502", subfields=new_subs)

    def update_pagenumber(self):
        """300 page number."""
        pages = record_get_field_instances(self.record, '300')
        for field in pages:
            for idx, (key, value) in enumerate(field[0]):
                if key == 'a':
                    field[0][idx] = ('a', "{0} p".format(value))

    def update_date(self):
        """269 Date normalization."""
        dates_269 = record_get_field_instances(self.record, '269')
        for idx, field in enumerate(dates_269):
            new_subs = []
            old_subs = field[0]
            for code, value in old_subs:
                if code == "c":
                    new_subs.append((
                        "c",
                        convert_date_from_iso_to_human(value)
                    ))
                else:
                    new_subs.append((code, value))
            dates_269[idx] = field_swap_subfields(field, new_subs)

    def update_date_year(self):
        """260 Date normalization."""
        dates = record_get_field_instances(self.record, '260')
        for field in dates:
            for idx, (key, value) in enumerate(field[0]):
                if key == 'c':
                    field[0][idx] = ('c', value[:4])
                elif key == 't':
                    del field[0][idx]
        if not dates:
            published_years = record_get_field_values(self.record, "773", code="y")
            if published_years:
                record_add_field(
                    self.record, "260", subfields=[("c", published_years[0][:4])])
            else:
                other_years = record_get_field_values(self.record, "269", code="c")
                if other_years:
                    record_add_field(
                        self.record, "260", subfields=[("c", other_years[0][:4])])

    def is_published(self):
        """Check fields 980 and 773 to see if the record has already been published.

        :return: True is published, else False
        """
        field773 = record_get_field_instances(self.record, '773')
        for f773 in field773:
            if 'c' in field_get_subfields(f773):
                return True
        return False

    def update_links_and_ffts(self):
        """FFT (856) Dealing with files."""
        for field in record_get_field_instances(self.record,
                                                tag='856',
                                                ind1='4'):
            subs = field_get_subfields(field)
            newsubs = []
            url = subs.get("u", [])

            if not url:
                record_delete_field(self.record, '856', ind1='4',
                                    field_position_global=field[4])
                continue
            url = url[0]
            if "inspirehep.net/record" in url and url.endswith("pdf"):
                # We have an FFT from INSPIRE
                newsubs.append(('a', url))
                description = subs.get("y", [])
                if description:
                    newsubs.append(('d', description[0]))
                if newsubs:
                    record_add_field(self.record, 'FFT', subfields=newsubs)
                    record_delete_field(self.record, '856', ind1='4',
                                        field_position_global=field[4])
            else:
                # Remove $w
                for idx, (key, value) in enumerate(field[0]):
                    if key == 'w':
                        del field[0][idx]

    def update_languages(self):
        """041 Language."""
        language_fields = record_get_field_instances(self.record, '041')
        language = "eng"
        record_delete_fields(self.record, "041")
        for field in language_fields:
            subs = field_get_subfields(field)
            if 'a' in subs:
                language = self.get_config_item(subs['a'][0], "languages")
                break
        new_subs = [('a', language)]
        record_add_field(self.record, "041", subfields=new_subs)



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2014, 2016 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import sys

from datetime import datetime

from harvestingkit.minidom_utils import (get_value_in_tag,
                                         xml_to_text)
from harvestingkit.utils import collapse_initials, safe_title
from harvestingkit.bibrecord import (
    record_add_field,
    create_record,
)


class PosPackage(object):
    """ This class is specialized in parsing xml records from
    PoS and create the corresponding Bibrecord object. """

    def _get_authors(self):
        authors = []
        for pextag in self.document.getElementsByTagName('pex-dc:creator'):
            affiliations = []
            for auttag in pextag.getElementsByTagName('pex-dc:name'):
                author = xml_to_text(auttag)
                lastname = author.split()[-1]
                givenames = " ".join(author.split()[:-1])
                givenames = collapse_initials(givenames)
                name = "%s, %s" % (lastname, givenames)
                name = safe_title(name)
                for afftag in pextag.getElementsByTagName('pex-dc:affiliation'):
                    if afftag:
                        affiliations.append(xml_to_text(afftag))
                authors.append((name, affiliations))
        return authors

    def _get_title(self):
        try:
            return get_value_in_tag(self.document, 'pex-dc:title')
        except Exception:
            print >> sys.stderr, "Can't find title"
            return ''

    def _get_language(self):
        try:
            return get_value_in_tag(self.document, 'pex-dc:language')
        except Exception:
            print >> sys.stderr, "Can't find language"
            return ''

    def _get_publisher(self):
        try:
            publisher = get_value_in_tag(self.document, 'pex-dc:publisher')
            if publisher == 'Sissa Medialab':
                publisher = 'SISSA'
            return publisher
        except Exception:
            print >> sys.stderr, "Can't find publisher"
            return ''

    def _get_date(self):
        try:
            date = get_value_in_tag(self.document, 'pex-dc:date')
            if len(date) == 20:
                date = datetime.strptime(date, '%Y-%m-%dT%H:%M:%SZ')
                date = date.strftime("%Y-%m-%d")
                date = str(date)
            return date
        except Exception:
            print >> sys.stderr, "Can't find date"
            return ''

    def _get_copyright(self):
        try:
            record_copyright = get_value_in_tag(self.document, 'pex-dc:rights')
            if record_copyright == 'Creative Commons Attribution-NonCommercial-ShareAlike':
                record_copyright = 'CC-BY-NC-SA'
            return record_copyright
        except Exception:
            print >> sys.stderr, "Can't find copyright"
            return ''

    def _get_subject(self):
        try:
            return get_value_in_tag(self.document, 'pex-dc:subject')
        except Exception:
            print >> sys.stderr, "Can't find subject"
            return ''

    def get_identifier(self):
        """ Returns the identifier of the paper corresponding
            to this record containing the conference which it
            was published and the proceeding number."""
        try:
            return get_value_in_tag(self.document, 'identifier')
        except Exception:
            print >> sys.stderr, "Can't find identifier"
            return ''

    def get_record(self, record):
        """ Reads a dom xml element in oaidc format and
            returns the bibrecord object """
        self.document = record
        rec = create_record()
        language = self._get_language()
        if language and language != 'en':
            record_add_field(rec, '041', subfields=[('a', language)])
        publisher = self._get_publisher()
        date = self._get_date()
        if publisher and date:
            record_add_field(rec, '260', subfields=[('b', publisher),
                                                    ('c', date)])
        elif publisher:
            record_add_field(rec, '260', subfields=[('b', publisher)])
        elif date:
            record_add_field(rec, '260', subfields=[('c', date)])
        title = self._get_title()
        if title:
            record_add_field(rec, '245', subfields=[('a', title)])
        record_copyright = self._get_copyright()
        if record_copyright:
            record_add_field(rec, '540', subfields=[('a', record_copyright)])
        subject = self._get_subject()
        if subject:
            record_add_field(rec, '650', ind1='1', ind2='7', subfields=[('a', subject),
                                                                        ('2', 'PoS')])
        authors = self._get_authors()
        first_author = True
        for author in authors:
            subfields = [('a', author[0])]
            for affiliation in author[1]:
                subfields.append(('v', affiliation))
            if first_author:
                record_add_field(rec, '100', subfields=subfields)
                first_author = False
            else:
                record_add_field(rec, '700', subfields=subfields)
        identifier = self.get_identifier()
        conference = identifier.split(':')[2]
        conference = conference.split('/')[0]
        contribution = identifier.split(':')[2]
        contribution = contribution.split('/')[1]
        record_add_field(rec, '773', subfields=[('p', 'PoS'),
                                                ('v', conference.replace(' ', '')),
                                                ('c', contribution),
                                                ('y', date[:4])])
        record_add_field(rec, '980', subfields=[('a', 'ConferencePaper')])
        record_add_field(rec, '980', subfields=[('a', 'HEP')])
        return rec



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Contains parsers for MARCXML flavors."""

from __future__ import print_function

from tempfile import mkdtemp

from ..bibrecord import (record_get_field_instances,
                         record_add_field,
                         record_delete_fields,
                         record_strip_controlfields,
                         record_xml_output,
                         field_get_subfields,
                         BibRecordPackage)

from ..utils import create_logger


class MARCXMLConversion(object):

    """Convert a BibRecord from a MARCXML mapping to another."""

    logger = create_logger("harvestingkit.MARCXMLConversion")
    kbs = {}

    def __init__(self, bibrec, strip_fields_list=None):
        """Create an instance of a record conversion."""
        self.record = bibrec
        self.strip_fields_list = strip_fields_list
        self.hidden = False

    @classmethod
    def convert_all(cls, records):
        """Convert the list of bibrecs into one MARCXML.

        >>> from harvestingkit.bibrecord import BibRecordPackage
        >>> from harvestingkit.inspire_cds_package import Inspire2CDS
        >>> bibrecs = BibRecordPackage("inspire.xml")
        >>> bibrecs.parse()
        >>> xml = Inspire2CDS.convert_all(bibrecs.get_records())

        :param records: list of BibRecord dicts
        :type records: list

        :returns: MARCXML as string
        """
        out = ["<collection>"]
        for rec in records:
            conversion = cls(rec)
            out.append(conversion.convert())
        out.append("</collection>")
        return "\n".join(out)

    @classmethod
    def from_source(cls, source):
        """Yield single conversion objects from a MARCXML file or string.

        >>> from harvestingkit.inspire_cds_package import Inspire2CDS
        >>> for record in Inspire2CDS.from_source("inspire.xml"):
        >>>     xml = record.convert()

        """
        bibrecs = BibRecordPackage(source)
        bibrecs.parse()
        for bibrec in bibrecs.get_records():
            yield cls(bibrec)

    @classmethod
    def get_config_item(cls, key, kb_name, allow_substring=True):
        """Return the opposite mapping by searching the imported KB."""
        config_dict = cls.kbs.get(kb_name, None)
        if config_dict:
            if key in config_dict:
                return config_dict[key]
            elif allow_substring:
                res = [v for k, v in config_dict.items() if key in k]
                if res:
                    return res[0]
        return key

    @staticmethod
    def load_config(from_key, to_key):
        """Load configuration from config.

        Meant to run only once per system process as
        class variable in subclasses."""
        from .mappings import mappings
        kbs = {}
        for key, values in mappings['config'].iteritems():
            parse_dict = {}
            for mapping in values:
                # {'inspire': 'Norwegian', 'cds': 'nno'}
                # -> {"Norwegian": "nno"}
                parse_dict[mapping[from_key]] = mapping[to_key]
            kbs[key] = parse_dict
        return kbs

    def convert(self):
        """Convert current record and return MARCXML.

        >>> from harvestingkit.bibrecord import BibRecordPackage
        >>> from harvestingkit.inspire_cds_package import Inspire2CDS
        >>> bibrecs = BibRecordPackage("inspire.xml")
        >>> bibrecs.parse()
        >>> for bibrec in bibrecs:
        >>>     rec = Inspire2CDS(bibrec)
        >>>     xml = rec.convert(bibrec)

        :returns: MARCXML as string
        """
        self.get_record()
        return self.get_xml()

    def get_record(self):
        """Return the record."""
        return self.record

    def get_xml(self):
        """Return the current record as MARCXML."""
        return record_xml_output(self.record)

    def get_recid(self):
        """Return the record ID from 001."""
        try:
            return self.record['001'][0][3]
        except KeyError:
            return

    def get_local_folder(self):
        """Return a path to a generated local folder."""
        if not hasattr(self, "local_folder"):
            self.local_folder = mkdtemp()
        return self.local_folder

    def match(self, query=None, **kwargs):
        """Try to match the current record to the database."""
        from invenio.search_engine import perform_request_search
        if not query:
            # We use default setup
            recid = self.record["001"][0][3]
            return perform_request_search(p="035:%s" % (recid,),
                                          of="id")
        else:
            if "recid" not in kwargs:
                kwargs["recid"] = self.record["001"][0][3]
            return perform_request_search(p=query % kwargs,
                                          of="id")

    def remove_controlfields(self):
        """Clear any existing control fields."""
        record_strip_controlfields(self.record)

    def keep_only_fields(self):
        """Keep only fields listed in field_list."""
        for tag in self.record.keys():
            if tag not in self.fields_list:
                record_delete_fields(self.record, tag)

    def strip_fields(self):
        """Clear any fields listed in field_list."""
        for tag in self.record.keys():
            if tag in self.fields_list:
                record_delete_fields(self.record, tag)

    def add_systemnumber(self, source, recid=None):
        """Add 035 number from 001 recid with given source."""
        if not recid:
            recid = self.get_recid()
        if not self.hidden and recid:
            record_add_field(
                self.record,
                tag='035',
                subfields=[('9', source), ('a', recid)]
            )

    def add_control_number(self, tag, value):
        """Add a control-number 00x for given tag with value."""
        record_add_field(self.record,
                         tag,
                         controlfield_value=value)

    def update_subject_categories(self, primary, secondary, kb):
        """650 Translate Categories."""
        category_fields = record_get_field_instances(self.record,
                                                     tag='650',
                                                     ind1='1',
                                                     ind2='7')
        record_delete_fields(self.record, "650")
        for field in category_fields:
            for idx, (key, value) in enumerate(field[0]):
                if key == 'a':
                    new_value = self.get_config_item(value, kb)
                    if new_value != value:
                        new_subs = [('2', secondary), ('a', new_value)]
                    else:
                        new_subs = [('2', primary), ('a', value)]
                    record_add_field(self.record, "650", ind1="1", ind2="7",
                                     subfields=new_subs)
                    break

    def update_languages(self):
        """041 Language."""
        language_fields = record_get_field_instances(self.record, '041')
        record_delete_fields(self.record, "041")
        for field in language_fields:
            subs = field_get_subfields(field)
            if 'a' in subs:
                if "eng" in subs['a']:
                    continue
                new_value = self.get_config_item(subs['a'][0], "languages")
                new_subs = [('a', new_value)]
                record_add_field(self.record, "041", subfields=new_subs)



# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
from __future__ import print_function

import sys
import os
import time

from ftplib import FTP, error_perm
from os.path import join
from os import remove, getcwd
from urlparse import urlparse
from netrc import netrc
from datetime import datetime


class FtpHandler(object):
    """ This class provides an interface to easily connect to an FTP server,
    list its contents and download files/folders.

    :param server: the URL to access the FTP server.
    :type server: string
    :param username: the user-name used to connect to the server.
    :type username: string
    :param passwd: the password used to connect to the server.
    :type passwd: string
    :param netrc_file: path to a netrc file that can be used
                               for authentication with the server.
    :type netrc_file: string
    """
    def __init__(self, server, username='', passwd='', netrc_file=''):
        server = urlparse(server)
        if server.netloc:
            server = server.netloc
        elif server.path:
            server = server.path
        self._ftp = FTP(server)
        self._username = username
        self._passwd = passwd
        if netrc_file:
            logininfo = netrc(netrc_file).authenticators(server)
            self._username, _, self._passwd = logininfo
        self.connect()
        self._home = self._ftp.pwd()

    def connect(self):
        """ Connects and logins to the server. """
        self._ftp.connect()
        self._ftp.login(user=self._username, passwd=self._passwd)

    def close(self):
        """ Closes the connection to the server. """
        self._ftp.close()

    def download_folder(self, folder='', target_folder=''):
        """ Downloads a whole folder from the server.
        FtpHandler.download_folder() will download all the files
        from the server in the working directory.

        :param folder: the absolute path for the folder on the server.

        :type folder: string
        :param target_folder: absolute or relative path for the
                              destination folder default is the
                              working directory.
        :type target_folder: string
        """
        files, folders = self.ls(folder)
        for fl in files:
            self.download(join(folder, fl), target_folder)
        for fld in folders:
            self.download_folder(join(folder, fld), target_folder)

    def download(self, source_file, target_folder=''):
        """ Downloads a file from the FTP server to target folder

        :param source_file: the absolute path for the file on the server
                   it can be the one of the files coming from
                   FtpHandler.dir().
        :type source_file: string
        :param target_folder: relative or absolute path of the
                              destination folder default is the
                              working directory.
        :type target_folder: string
        """
        current_folder = self._ftp.pwd()

        if not target_folder.startswith('/'):  # relative path
            target_folder = join(getcwd(), target_folder)

        folder = os.path.dirname(source_file)
        self.cd(folder)

        if folder.startswith("/"):
            folder = folder[1:]

        destination_folder = join(target_folder, folder)
        if not os.path.exists(destination_folder):
            print("Creating folder", destination_folder)
            os.makedirs(destination_folder)

        source_file = os.path.basename(source_file)
        destination = join(destination_folder, source_file)
        try:
            with open(destination, 'wb') as result:
                self._ftp.retrbinary('RETR %s' % (source_file,),
                                     result.write)
        except error_perm as e:  # source_file is a folder
            print(e)
            remove(join(target_folder, source_file))
            raise
        self._ftp.cwd(current_folder)

    def cd(self, folder):
        """ Changes the working directory on the server.

        :param folder: the desired directory.
        :type folder: string
        """
        if folder.startswith('/'):
            self._ftp.cwd(folder)
        else:
            for subfolder in folder.split('/'):
                if subfolder:
                    self._ftp.cwd(subfolder)

    def ls(self, folder=''):
        """ Lists the files and folders of a specific directory
        default is the current working directory.

        :param folder: the folder to be listed.
        :type folder: string

        :returns: a tuple with the list of files in the folder
                  and the list of subfolders in the folder.
        """
        current_folder = self._ftp.pwd()
        self.cd(folder)
        contents = []
        self._ftp.retrlines('LIST', lambda a: contents.append(a))
        files = filter(lambda a: a.split()[0].startswith('-'), contents)
        folders = filter(lambda a: a.split()[0].startswith('d'), contents)
        files = map(lambda a: ' '.join(a.split()[8:]), files)
        folders = map(lambda a: ' '.join(a.split()[8:]), folders)
        self._ftp.cwd(current_folder)
        return files, folders

    def dir(self, folder='', prefix=''):
        """ Lists all the files on the folder given as parameter.
        FtpHandler.dir() lists all the files on the server.

        :para folder: the folder to be listed.
        :type folder: string

        :param prefix: it does not belong to the interface,
                       it is used to recursively list the subfolders.

        :returns: a list with all the files in the server.
        """
        files, folders = self.ls(folder)
        result = files
        inner = []
        for fld in folders:
            try:
                inner += self.dir(folder + '/' + fld, prefix + fld + '/')
            except:
                pass
        result += inner
        if prefix:
            result = map(lambda a: prefix + a, result)
        return result

    def mkdir(self, folder):
        """ Creates a folder in the server

        :param folder: the folder to be created.
        :type folder: string
        """
        current_folder = self._ftp.pwd()
        #creates the necessary folders on
        #the server if they don't exist
        folders = folder.split('/')
        for fld in folders:
            try:
                self.cd(fld)
            except error_perm:  # folder does not exist
                self._ftp.mkd(fld)
                self.cd(fld)
        self.cd(current_folder)

    def rm(self, filename):
        """ Delete a file from the server.

        :param filename: the file to be deleted.
        :type filename: string
        """
        try:
            self._ftp.delete(filename)
        except error_perm:  # target is either a directory
                            # either it does not exist
            try:
                current_folder = self._ftp.pwd()
                self.cd(filename)
            except error_perm:
                print('550 Delete operation failed %s '
                      'does not exist!' % (filename,))
            else:
                self.cd(current_folder)
                print('550 Delete operation failed %s '
                      'is a folder. Use rmdir function '
                      'to delete it.' % (filename,))

    def rmdir(self, foldername):
        """ Delete a folder from the server.

        :param foldername: the folder to be deleted.
        :type foldername: string
        """
        current_folder = self._ftp.pwd()
        try:
            self.cd(foldername)
        except error_perm:
            print('550 Delete operation failed folder %s '
                  'does not exist!' % (foldername,))
        else:
            self.cd(current_folder)
            try:
                self._ftp.rmd(foldername)
            except error_perm:  # folder not empty
                self.cd(foldername)
                contents = self.ls()
                #delete the files
                map(self._ftp.delete, contents[0])
                #delete the subfolders
                map(self.rmdir, contents[1])
                self.cd(current_folder)
                self._ftp.rmd(foldername)

    def get_filesize(self, filename):
        """ Returns the filesize of a file

        :param filename: the full path to the file on the server.
        :type filename: string

        :returns: string representation of the filesize.
        """
        result = []

        def dir_callback(val):
            result.append(val.split()[4])

        self._ftp.dir(filename, dir_callback)
        return result[0]

    def get_datestamp(self, filename):
        datestamp = self._ftp.sendcmd('MDTM ' + filename)
        datestamp = datetime.strptime(datestamp[4:],
                                      "%Y%m%d%H%M%S").strftime("%Y-%M-%d")
        return datestamp

    def check_pkgs_integrity(self, filelist, logger,
                             timeout=120, sleep_time=10):
        """ Checks if files are not being uploaded to server.

        :param filelist - a list of filenames to check.
        :type filelist: list
        :param timeout - time after which the script will register an error.
        :type timeout: int
        """
        ref_1 = []
        ref_2 = []
        i = 1
        print("\nChecking packages integrity.")
        for filename in filelist:
            ref_1.append(self.get_filesize(filename))
        print("\nGoing to sleep for %i sec." % (sleep_time,))
        time.sleep(sleep_time)

        while sleep_time * i < timeout:
            for filename in filelist:
                ref_2.append(self.get_filesize(filename))
            if ref_1 == ref_2:
                print(sys.stdout, "\nIntegrity OK:)")
                logger.info("Packages integrity OK.")
                break
            else:
                print("\nWaiting %d time for integrity..." % (i,))
                logger.info("\nWaiting %d time for integrity..." % (i,))
                i += 1
                ref_1, ref_2 = ref_2, []
                time.sleep(sleep_time)
        else:
            not_finished_files = []
            for count, val1 in enumerate(ref_1):
                if val1 != ref_2[count]:
                    not_finished_files.append(filelist[count])

            print("\nOMG, OMG something wrong with integrity.")
            logger.error("Integrity check failed for files %s"
                         % (not_finished_files,))

    def upload(self, filename, location=''):
        """ Uploads a file on the server to the desired location

        :param filename: the name of the file to be uploaded.
        :type filename: string
        :param location: the directory in which the file will
                         be stored.
        :type location: string
        """
        current_folder = self._ftp.pwd()
        self.mkdir(location)
        self.cd(location)
        fl = open(filename, 'rb')
        filename = filename.split('/')[-1]
        self._ftp.storbinary('STOR %s' % filename, fl)
        fl.close()
        self.cd(current_folder)



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2014, 2015, 2016, 2017, 2019 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Utility functions for Harvesting Kit."""

import re
import os
import pkg_resources
import htmlentitydefs
import requests
import subprocess
import logging
import fnmatch
import zipfile

from datetime import datetime
from tempfile import mkdtemp, mkstemp
from lxml import etree
from unidecode import unidecode

from .config import COMMON_ACRONYMS, OA_LICENSES


def make_user_agent(component=None):
    """ create string suitable for HTTP User-Agent header """
    packageinfo = pkg_resources.require("harvestingkit")[0]
    useragent = "{0}/{1}".format(packageinfo.project_name, packageinfo.version)
    if component is not None:
        useragent += " {0}".format(component)
    return useragent


def create_record():
    """Return a new XML document."""
    return etree.Element("record")


def record_add_field(rec, tag, ind1='', ind2='', subfields=[],
                     controlfield_value=''):
    """Add a MARCXML datafield as a new child to a XML document."""
    if controlfield_value:
        doc = etree.Element("controlfield",
                            attrib={
                                "tag": tag,
                            })
        doc.text = unicode(controlfield_value)
    else:
        doc = etree.Element("datafield",
                            attrib={
                                "tag": tag,
                                "ind1": ind1,
                                "ind2": ind2,
                            })
        for code, value in subfields:
            field = etree.SubElement(doc, "subfield", attrib={"code": code})
            field.text = value
    rec.append(doc)
    return rec


def record_xml_output(rec, pretty=True):
    """Given a document, return XML prettified."""
    from .html_utils import MathMLParser
    ret = etree.tostring(rec, xml_declaration=False)

    # Special MathML handling
    ret = re.sub("(&lt;)(([\/]?{0}))".format("|[\/]?".join(MathMLParser.mathml_elements)), '<\g<2>', ret)
    ret = re.sub("&gt;", '>', ret)
    if pretty:
        # We are doing our own prettyfication as etree pretty_print is too insane.
        ret = ret.replace('</datafield>', '  </datafield>\n')
        ret = re.sub(r'<datafield(.*?)>', r'  <datafield\1>\n', ret)
        ret = ret.replace('</subfield>', '</subfield>\n')
        ret = ret.replace('<subfield', '    <subfield')
        ret = ret.replace('record>', 'record>\n')
    return ret


def escape_for_xml(data, tags_to_keep=None):
    """Transform & and < to XML valid &amp; and &lt.

    Pass a list of tags as string to enable replacement of
    '<' globally but keep any XML tags in the list.
    """
    data = re.sub("&", "&amp;", data)
    if tags_to_keep:
        data = re.sub(r"(<)(?![\/]?({0})\b)".format("|".join(tags_to_keep)), '&lt;', data)
    else:
        data = re.sub("<", "&lt;", data)
    return data


def unescape(text):
    """Remove HTML or XML character references and entities from a text string.

    NOTE: Does not remove &amp; &lt; and &gt;.

    @param text The HTML (or XML) source text.
    @return The plain text, as a Unicode string, if necessary.
    """
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        elif text[1:-1] not in ("gt", "lt", "amp"):
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text  # leave as is
    return re.sub("&#?\w+;", fixup, text)


def format_arxiv_id(arxiv_id):
    """Properly format arXiv IDs."""
    if arxiv_id and "/" not in arxiv_id and "arXiv" not in arxiv_id:
        return "arXiv:%s" % (arxiv_id,)
    elif arxiv_id and '.' not in arxiv_id and arxiv_id.lower().startswith('arxiv:'):
        return arxiv_id[6:]  # strip away arxiv: for old identifiers
    else:
        return arxiv_id


def safe_title(text):
    """Perform a UTF-8 safe str.title() wrapper."""
    return unicode(text, "utf-8").title().encode("utf-8")


def collapse_initials(name):
    """Remove the space between initials, eg T. A. --> T.A."""
    if len(name.split(".")) > 1:
        name = re.sub(r'([A-Z]\.)[\s\-]+(?=[A-Z]\.)', r'\1', name)
    return name


def fix_journal_name(journal, knowledge_base):
    """Convert journal name to Inspire's short form."""
    if not journal:
        return '', ''
    if not knowledge_base:
        return journal, ''
    if len(journal) < 2:
        return journal, ''
    volume = ''
    if (journal[-1] <= 'Z' and journal[-1] >= 'A') \
            and (journal[-2] == '.' or journal[-2] == ' '):
        volume += journal[-1]
        journal = journal[:-1]
    journal = journal.strip()

    if journal.upper() in knowledge_base:
        journal = knowledge_base[journal.upper()].strip()
    elif journal in knowledge_base:
        journal = knowledge_base[journal].strip()
    elif '.' in journal:
        journalnodots = journal.replace('. ', ' ')
        journalnodots = journalnodots.replace('.', ' ').strip().upper()
        if journalnodots in knowledge_base:
            journal = knowledge_base[journalnodots].strip()

    journal = journal.replace('. ', '.')
    return journal, volume


def add_nations_field(authors_subfields):
    """Add correct nations field according to mapping in NATIONS_DEFAULT_MAP."""
    from .config import NATIONS_DEFAULT_MAP
    result = []
    for field in authors_subfields:
        if field[0] == 'v':
            values = [x.replace('.', '') for x in field[1].split(', ')]
            possible_affs = filter(lambda x: x is not None,
                                   map(NATIONS_DEFAULT_MAP.get, values))
            if 'CERN' in possible_affs and 'Switzerland' in possible_affs:
                # Don't use remove in case of multiple Switzerlands
                possible_affs = [x for x in possible_affs
                                 if x != 'Switzerland']

            result.extend(possible_affs)

    result = sorted(list(set(result)))

    if result:
        authors_subfields.extend([('w', res) for res in result])
    else:
        authors_subfields.append(('w', 'HUMAN CHECK'))


def fix_dashes(string):
    """Fix bad Unicode special dashes in string."""
    string = string.replace(u'\u05BE', '-')
    string = string.replace(u'\u1806', '-')
    string = string.replace(u'\u2E3A', '-')
    string = string.replace(u'\u2E3B', '-')
    string = unidecode(string)
    return re.sub(r'--+', '-', string)


def fix_title_capitalization(title):
    """Try to capitalize properly a title string."""
    if re.search("[A-Z]", title) and re.search("[a-z]", title):
        return title
    word_list = re.split(' +', title)
    final = [word_list[0].capitalize()]
    for word in word_list[1:]:
        if word.upper() in COMMON_ACRONYMS:
            final.append(word.upper())
        elif len(word) > 3:
            final.append(word.capitalize())
        else:
            final.append(word.lower())
    return " ".join(final)


def convert_html_subscripts_to_latex(text):
    """Convert some HTML tags to latex equivalents."""
    text = re.sub("<sub>(.*?)</sub>", r"$_{\1}$", text)
    text = re.sub("<sup>(.*?)</sup>", r"$^{\1}$", text)
    return text


def download_file(from_url, to_filename=None,
                  chunk_size=1024 * 8, retry_count=3):
    """Download URL to a file."""
    if not to_filename:
        to_filename = get_temporary_file()

    session = requests.Session()
    adapter = requests.adapters.HTTPAdapter(max_retries=retry_count)
    session.mount(from_url, adapter)
    response = session.get(from_url, stream=True)
    with open(to_filename, 'wb') as fd:
        for chunk in response.iter_content(chunk_size):
            fd.write(chunk)
    return to_filename


def run_shell_command(commands, **kwargs):
    """Run a shell command."""
    p = subprocess.Popen(commands,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE,
                         **kwargs)
    output, error = p.communicate()
    return p.returncode, output, error


def create_logger(name,
                  filename=None,
                  logging_level=logging.DEBUG):
    """Create a logger object."""
    logger = logging.getLogger(name)
    formatter = logging.Formatter(('%(asctime)s - %(name)s - '
                                   '%(levelname)-8s - %(message)s'))

    if filename:
        fh = logging.FileHandler(filename=filename)
        fh.setFormatter(formatter)
        logger.addHandler(fh)

    ch = logging.StreamHandler()
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    logger.setLevel(logging_level)

    return logger


def unzip(zipped_file, output_directory=None,
          prefix="harvestingkit_unzip_", suffix=""):
    """Uncompress a zipped file from given filepath to an (optional) location.

    If no location is given, a temporary folder will be generated inside
    CFG_TMPDIR, prefixed with "apsharvest_unzip_".
    """
    if not output_directory:
        # We create a temporary directory to extract our stuff in
        try:
            output_directory = mkdtemp(suffix=suffix,
                                       prefix=prefix)
        except Exception, e:
            try:
                os.removedirs(output_directory)
            except TypeError:
                pass
            raise e
    return _do_unzip(zipped_file, output_directory)


def _do_unzip(zipped_file, output_directory):
    """Perform the actual uncompression."""
    z = zipfile.ZipFile(zipped_file)
    for path in z.namelist():
        relative_path = os.path.join(output_directory, path)
        dirname, dummy = os.path.split(relative_path)
        try:
            if relative_path.endswith(os.sep) and not os.path.exists(dirname):
                os.makedirs(relative_path)
            elif not os.path.exists(relative_path):
                dirname = os.path.join(output_directory, os.path.dirname(path))
                if os.path.dirname(path) and not os.path.exists(dirname):
                    os.makedirs(dirname)
                fd = open(relative_path, "w")
                fd.write(z.read(path))
                fd.close()
        except IOError, e:
            raise e
    return output_directory


def locate(pattern, root=os.curdir):
    """Locate all files matching supplied filename pattern recursively."""
    for path, dummy, files in os.walk(os.path.abspath(root)):
        for filename in fnmatch.filter(files, pattern):
            yield os.path.join(path, filename)


def punctuate_authorname(an):
    """Punctuate author names properly.

    Expects input in the form 'Bloggs, J K' and will return 'Bloggs, J. K.'.
    """
    name = an.strip()
    parts = [x for x in name.split(',') if x != '']
    ret_str = ''
    for idx, part in enumerate(parts):
        subparts = part.strip().split(' ')
        for sidx, substr in enumerate(subparts):
            ret_str += substr
            if len(substr) == 1:
                ret_str += '.'
            if sidx < (len(subparts) - 1):
                ret_str += ' '
        if idx < (len(parts) - 1):
            ret_str += ', '
    return ret_str.strip()


def convert_date_to_iso(value):
    """Convert a date-value to the ISO date standard."""
    date_formats = ["%d %b %Y", "%Y/%m/%d"]
    for dformat in date_formats:
        try:
            date = datetime.strptime(value, dformat)
            return date.strftime("%Y-%m-%d")
        except ValueError:
            pass
    return value


def convert_date_from_iso_to_human(value):
    """Convert a date-value to the ISO date standard for humans."""
    try:
        year, month, day = value.split("-")
    except ValueError:
        # Not separated by "-". Space?
        try:
            year, month, day = value.split(" ")
        except ValueError:
            # What gives? OK, lets just return as is
            return value

    try:
        date_object = datetime(int(year), int(month), int(day))
    except TypeError:
        return value
    return date_object.strftime("%d %b %Y")


def get_converted_image_name(image):
    """Return the name of the image after it has been converted to png format.

    Strips off the old extension.

    :param image: The fullpath of the image before conversion
    :type image: string

    :return: the fullpath of the image after convertion
    """
    png_extension = '.png'

    if image[(0 - len(png_extension)):] == png_extension:
        # it already ends in png!  we're golden
        return image

    img_dir = os.path.split(image)[0]
    image = os.path.split(image)[-1]

    # cut off the old extension
    if len(image.split('.')) > 1:
        old_extension = '.' + image.split('.')[-1]
        converted_image = image[:(0 - len(old_extension))] + png_extension

    else:
        converted_image = image + png_extension

    return os.path.join(img_dir, converted_image)


def convert_images(image_list):
    """Convert list of images to PNG format.

    @param: image_list ([string, string, ...]): the list of image files
        extracted from the tarball in step 1

    @return: image_list ([str, str, ...]): The list of image files when all
        have been converted to PNG format.
    """
    png_output_contains = 'PNG image'
    ret_list = []
    for image_file in image_list:
        if os.path.isdir(image_file):
            continue

        dummy1, cmd_out, dummy2 = run_shell_command('file %s', (image_file,))
        if cmd_out.find(png_output_contains) > -1:
            ret_list.append(image_file)
        else:
            # we're just going to assume that ImageMagick can convert all
            # the image types that we may be faced with
            # for sure it can do EPS->PNG and JPG->PNG and PS->PNG
            # and PSTEX->PNG
            converted_image_file = get_converted_image_name(image_file)
            cmd_list = ['convert', image_file, converted_image_file]
            dummy1, cmd_out, cmd_err = run_shell_command(cmd_list)
            if cmd_err == '':
                ret_list.append(converted_image_file)
            else:
                raise Exception(cmd_err)
    return ret_list


def get_temporary_file(prefix="tmp_",
                       suffix="",
                       directory=None):
    """Generate a safe and closed filepath."""
    try:
        file_fd, filepath = mkstemp(prefix=prefix,
                                    suffix=suffix,
                                    dir=directory)
        os.close(file_fd)
    except IOError, e:
        try:
            os.remove(filepath)
        except Exception:
            pass
        raise e
    return filepath


def return_letters_from_string(text):
    """Get letters from string only."""
    out = ""
    for letter in text:
        if letter.isalpha():
            out += letter
    return out

def license_is_oa(license):
    """Return True if license is compatible with Open Access"""
    for oal in OA_LICENSES:
        if re.search(oal, license):
            return True
    return False




# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2013, 2014, 2015 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import re
import sys
import time
from os import pardir
from os.path import (join,
                     dirname,
                     basename)
try:
    from invenio.errorlib import register_exception
except ImportError:
    register_exception = lambda a=1, b=2: True
from harvestingkit.minidom_utils import (get_value_in_tag,
                                         xml_to_text)
from harvestingkit.utils import (format_arxiv_id,
                                 add_nations_field)
from harvestingkit.bibrecord import (
    record_add_field,
    create_record,
    record_xml_output,
)
from xml.dom.minidom import parse


class JATSParser(object):
    def __init__(self, tag_to_remove=None, extract_nations=False):
        self.references = None
        self.tag_to_remove = tag_to_remove
        self.extract_nations = extract_nations

    def get_article(self, path):
        return parse(open(path))

    def get_title(self, xml):
        try:
            return get_value_in_tag(xml, "article-title", tag_to_remove=self.tag_to_remove)
        except Exception:
            print >> sys.stderr, "Can't find title"

    def get_issn(self, xml):
        issns = xml.getElementsByTagName('issn')
        ret = None

        for issn in issns:
            if issn.getAttribute("pub-type").encode('utf-8') == 'epub':
                ret = issn.getAttribute("pub-type").encode('utf-8')

        if not ret and issns:
            ret = xml_to_text(issns[0])

        return ret

    def get_date(self, xml):
        dates = xml.getElementsByTagName('pub-date')
        ret = None
        for date in dates:
            if date.getAttribute('pub-type').encode('utf-8') == 'epub':
                ret = get_value_in_tag(date, 'year')

        if not ret and dates:
            return dates[0]
        else:
            return ret

    def get_publication_information(self, xml):
        jid = get_value_in_tag(xml, "journal-title")
        journal = ""
        if "European Physical Journal" in jid:
            journal = "EPJC"

        try:
            art = xml.getElementsByTagName('article-meta')[0]
        except IndexError as err:
            register_exception()
            print >> sys.stderr, "ERROR: XML corrupted: %s" % err
            pass
        except Exception as err:
            register_exception()
            print >> sys.stderr, "ERROR: Exception captured: %s" % err
            pass

        issn = self.get_issn(art)
        volume = get_value_in_tag(art, "volume")
        issue = get_value_in_tag(art, "issue")
        year = self.get_date(art)
        first_page = get_value_in_tag(art, "fpage")
        last_page = get_value_in_tag(art, "lpage")
        doi = self.get_doi(art)

        return (journal, issn, volume, issue, first_page, last_page, year, doi)

    def get_doi(self, xml):
        ids = xml.getElementsByTagName('article-id')
        ret = ""
        for i in ids:
            if i.getAttribute('pub-id-type').encode('utf-8') == 'doi':
                ret = xml_to_text(i)

        if not ret:
            print >> sys.stdout, "Can't find DOI."
        return ret

    def _get_orcid(self, xml_author):
        try:
            contrib_id = xml_author.getElementsByTagName('contrib-id')[0]
            if contrib_id.getAttribute('contrib-id-type') == 'orcid':
                orcid_raw = xml_to_text(contrib_id)
                pattern = '\d\d\d\d-\d\d\d\d-\d\d\d\d-\d\d\d[\d|X]'
                return re.search(pattern, orcid_raw).group()
        except (IndexError, AttributeError):
            return None

    def get_authors(self, xml):
        authors = []
        for author in xml.getElementsByTagName("contrib"):
            # Springer puts colaborations in additional "contrib" tag so to
            # avoid having fake author with all affiliations we skip "contrib"
            # tag with "contrib" subtags.
            if author.getElementsByTagName("contrib"):
                continue
            tmp = {}
            surname = get_value_in_tag(author, "surname")
            if surname:
                tmp["surname"] = surname
            given_name = get_value_in_tag(author, "given-names")
            if given_name:
                tmp["given_name"] = given_name.replace('\n', ' ')
            if not surname and not given_name:
                tmp["name"] = get_value_in_tag(author, "string-name")
            # It's not there yet
            orcid = self._get_orcid(author)
            if orcid:
                tmp["orcid"] = 'ORCID:{0}'.format(orcid)

            # cross_refs = author.getElementsByTagName("ce:cross-ref")
            # if cross_refs:
            #     tmp["cross_ref"] = []
            #     for cross_ref in cross_refs:
            #         tmp["cross_ref"].append(cross_ref.getAttribute("refid").encode('utf-8'))
            tmp["affiliations_ids"] = []
            tmp["contact_ids"] = []

            xrefs = author.getElementsByTagName("xref")
            for x in xrefs:
                if x.getAttribute('ref-type').encode('utf-8') == 'aff':
                    tmp["affiliations_ids"].extend([a.encode('utf-8') for a in x.getAttribute('rid').split()])
                if x.getAttribute('ref-type').encode('utf-8') == 'corresp':
                    tmp["contact_ids"].extend([a.encode('utf-8') for a in x.getAttribute('rid').split()])

            authors.append(tmp)

        affiliations = {}
        for affiliation in xml.getElementsByTagName("aff"):
            aff_id = affiliation.getAttribute("id").encode('utf-8')
            # removes numbering in from affiliations
            text = re.sub(r'^(\d+,\ ?)', "", xml_to_text(affiliation, delimiter=", "))
            affiliations[aff_id] = text

        emails = {}
        for contact in xml.getElementsByTagName("corresp"):
            contact_id = contact.getAttribute("id").encode('utf-8')
            if contact.getElementsByTagName('email'):
                text = xml_to_text(contact.getElementsByTagName('email')[0])
                emails[contact_id] = text

        implicit_affilations = True
        for author in authors:
            matching_ref = [ref for ref in author.get("affiliations_ids") if ref in affiliations]
            if matching_ref:
                implicit_affilations = False
                author["affiliation"] = []
                for i in xrange(0, len(matching_ref)):
                    author["affiliation"].append(affiliations[matching_ref[i]])
            matching_contact = [cont for cont in author.get('contact_ids') if cont in emails]
            if matching_contact:
                author["email"] = emails[matching_contact[0]]

        if implicit_affilations and len(affiliations) > 1:
            print >> sys.stderr, "Implicit affiliations are used, but there are more than one affiliation: %s" % affiliations
        if implicit_affilations and len(affiliations) >= 1:
            for author in authors:
                author["affiliation"] = []
                for aff in affiliations.values():
                    author["affiliation"].append(aff)
        return authors

    def get_abstract(self, xml):
        try:
            return get_value_in_tag(xml, "abstract", tag_to_remove=self.tag_to_remove).replace("Abstract", "", 1)
        except Exception:
            print >> sys.stderr, "Can't find abstract"

    def get_copyright(self, xml, logger=None):
        tags = ["copyright-holder", "copyright-statement"]
        for tag in tags:
            if tag is "copyright-holder":
                ret = get_value_in_tag(xml, tag)
                if not ret:
                    if logger:
                        logger.info("Can't find copyright, trying different tag.")
                    print >> sys.stderr, "Can't find copyright, trying different tag."
                else:
                    return ret
            else:
                ret = get_value_in_tag(xml, tag)
                if not ret:
                    if logger:
                        logger.info("Can't find copyright")
                    print >> sys.stderr, "Can't find copyright"
                else:
                    ret = ret.split('.')
                    return ret[0]

    def get_keywords(self, xml):
        try:
            kwd_groups = xml.getElementsByTagName('kwd-group')
            pacs = []
            other = []
            for kwd_group in kwd_groups:
                if kwd_group.getAttribute('kwd-group-type').encode('utf-8') == "pacs":
                    pacs = [xml_to_text(keyword, tag_to_remove=self.tag_to_remove) for keyword in kwd_group.getElementsByTagName("kwd")]
                else:
                    other = [xml_to_text(keyword, tag_to_remove=self.tag_to_remove) for keyword in kwd_group.getElementsByTagName("kwd")]
            return {"pacs": pacs, "other": other}
        except Exception:
            print >> sys.stderr, "Can't find keywords"

    def get_ref_link(self, xml, name):
        links = xml.getElementsByTagName('ext-link')
        ret = None
        for link in links:
            if name in link.getAttribute("xlink:href").encode('utf-8'):
                ret = xml_to_text(link).strip()
        if not ret:
            links = xml.getElementsByTagName('elocation-id')
            for link in links:
                if name in link.getAttribute("content-type").encode('utf-8'):
                    ret = xml_to_text(link).strip()
        return ret

    def get_page_count(self, xml):
        counts = xml.getElementsByTagName("counts")
        if counts:
            page_count = counts[0].getElementsByTagName("page-count")
            if page_count:
                return page_count[0].getAttribute("count").encode('utf-8')
            else:
                return None
        else:
            return None

    def get_publication_date(self, xml, logger=None):
        date_xmls = xml.getElementsByTagName('pub-date')
        day = None
        month = None
        year = None
        if date_xmls:
            for date_xml in date_xmls:
                if date_xml.hasAttribute('pub-type'):
                    if date_xml.getAttribute('pub-type') == "epub":
                        day = get_value_in_tag(date_xml, "day")
                        month = get_value_in_tag(date_xml, "month")
                        year = get_value_in_tag(date_xml, "year")
                if not year:
                    day = get_value_in_tag(date_xml, "day")
                    month = get_value_in_tag(date_xml, "month")
                    year = get_value_in_tag(date_xml, "year")
            if logger:
                logger.info('%s-%s-%s' % (year, month, day))
            return '%s-%s-%s' % (year, month, day)
        else:
            print >> sys.stderr, "Can't find publication date. Using 'today'."
            if logger:
                logger.info("Can't find publication date. Using 'today'.")
            return time.strftime('%Y-%m-%d')

    def get_references(self, xml):
        references = []
        for reference in xml.getElementsByTagName("ref"):
            plain_text = None
            try:
                ref_type = reference.getElementsByTagName('mixed-citation')[0]
                ref_type = ref_type.getAttribute('publication-type').encode('utf-8')
            except:
                ref_type = reference.getElementsByTagName('citation')[0]
                ref_type = ref_type.getAttribute('publication-type').encode('utf-8')
            label = get_value_in_tag(reference, "label").strip('.')
            authors = []
            for author in reference.getElementsByTagName("name"):
                given_name = get_value_in_tag(author, "given-names")
                surname = get_value_in_tag(author, "surname")
                if given_name:
                    name = "%s, %s" % (surname, given_name)
                else:
                    name = surname
                if name.strip().split() == []:
                    name = get_value_in_tag(author, "string-name")
                authors.append(name)
            doi_tag = reference.getElementsByTagName("pub-id")
            doi = ""
            for tag in doi_tag:
                if tag.getAttribute("pub-id-type") == "doi":
                    doi = xml_to_text(tag)
            issue = get_value_in_tag(reference, "issue")
            page = get_value_in_tag(reference, "fpage")
            page_last = get_value_in_tag(reference, "lpage")
            title = get_value_in_tag(reference, "source")
            volume = get_value_in_tag(reference, "volume")
            year = get_value_in_tag(reference, "year")
            ext_link = format_arxiv_id(self.get_ref_link(reference, "arxiv"))
            if ref_type != 'journal':
                try:
                    plain_text = get_value_in_tag(reference,
                                                  "mixed-citation",
                                                  tag_to_remove=self.tag_to_remove)
                except:
                    plain_text = get_value_in_tag(reference,
                                                  "citation",
                                                  tag_to_remove=self.tag_to_remove)
            references.append((label, authors, doi,
                               issue, page, page_last,
                               title, volume, year,
                               ext_link, plain_text))
        self.references = references

    def get_record(self, f_path, publisher=None, collection=None, logger=None):
        xml = self.get_article(f_path)
        rec = create_record()
        title = self.get_title(xml)
        if title:
            record_add_field(rec, '245', subfields=[('a', title)])

        record_add_field(rec, '260', subfields=[('c', self.get_publication_date(xml, logger))])
        journal, issn, volume, issue, first_page, last_page, year, doi = self.get_publication_information(xml)

        if logger:
            logger.info("Creating record: %s %s" % (join(f_path, pardir), doi))

        if doi:
            record_add_field(rec, '024', ind1='7', subfields=[('a', doi), ('2', 'DOI')])
        authors = self.get_authors(xml)
        first_author = True
        for author in authors:
            if author.get('surname'):
                subfields = [('a', '%s, %s' % (author.get('surname'), author.get('given_name') or author.get('initials', '')))]
            else:
                subfields = [('a', '%s' % (author.get('name', '')))]
            if 'orcid' in author:
                subfields.append(('j', author['orcid']))
            if 'affiliation' in author:
                for aff in author["affiliation"]:
                    subfields.append(('v', aff))

                if self.extract_nations:
                    add_nations_field(subfields)

            if author.get('email'):
                    subfields.append(('m', author['email']))
            if first_author:
                record_add_field(rec, '100', subfields=subfields)
                first_author = False
            else:
                record_add_field(rec, '700', subfields=subfields)

        page_count = self.get_page_count(xml)
        if page_count:
            record_add_field(rec, '300', subfields=[('a', page_count)])
        abstract = self.get_abstract(xml)
        if abstract:
            record_add_field(rec, '520', subfields=[('a', abstract), ('9', publisher)])
        record_add_field(rec, '540', subfields=[('a', 'CC-BY-3.0'), ('u', 'http://creativecommons.org/licenses/by/3.0/')])
        copyright = self.get_copyright(xml, logger)
        if copyright:
            record_add_field(rec, '542', subfields=[('f', copyright)])
        keywords = self.get_keywords(xml)
        if keywords['pacs']:
            for keyword in keywords['pacs']:
                record_add_field(rec, '084', ind1='1', subfields=[('a', keyword), ('9', 'PACS')])
        if keywords['other']:
            for keyword in keywords['other']:
                record_add_field(rec, '653', ind1='1', subfields=[('a', keyword), ('9', 'author')])
        if first_page or last_page:
            pages = '%s-%s' % (first_page, last_page)
        else:
            article_meta = xml.getElementsByTagName('article-meta')[0]
            pages = get_value_in_tag(article_meta, "elocation-id")

        subfields = filter(lambda x: x[1] and x[1] != '-', [('p', journal),
                                                            ('v', volume),
                                                            ('n', issue),
                                                            ('c', pages),
                                                            ('y', year)])
        record_add_field(rec, '773', subfields=subfields)

        self.get_references(xml)
        for label, authors, doi, issue, page, page_last, title, volume, year, ext_link, plain_text in self.references:
            subfields = []
            if doi:
                subfields.append(('a', doi))
            for author in authors:
                subfields.append(('h', author))
            if issue:
                subfields.append(('n', issue))
            if label:
                subfields.append(('o', label))
            if year:
                subfields.append(('y', year))
            if ext_link:
                subfields.append(('r', ext_link))
            # should we be strict about it?
            if title and volume and year and page:
                subfields.append(('s', '%s %s (%s) %s' % (title, volume, year, page)))
            elif not plain_text:
                subfields.append(('m', ('%s %s %s %s' % (title, volume, year, page))))
            if plain_text:
                subfields.append(('m', plain_text))
            if subfields:
                record_add_field(rec, '999', ind1='C', ind2='5', subfields=subfields)
        # record_add_field(rec, 'FFT', subfields=[('a', join(path, 'main.pdf'))])
        pdf_path = join(dirname(f_path), 'BodyRef', 'PDF', basename(f_path)[:-len('_nlm.xml')] + '.pdf')
        try:
            open(pdf_path)
            record_add_field(rec, 'FFT', subfields=[('a', pdf_path), ('n', 'main'), ('f', '.pdf;pdfa')])
        except:
            register_exception(alert_admin=True)
            logger.error("No PDF for paper: %s" % (doi,))
        record_add_field(rec, 'FFT', subfields=[('a', f_path), ('n', 'main')])
        extra_subfields = []
        if collection:
            extra_subfields.append(('a', collection))
        if publisher:
            extra_subfields.append(('b', publisher))
        record_add_field(rec, '980', subfields=extra_subfields)
        return record_xml_output(rec)
